"""Gui related helper functionality."""

# Import built-in modules
import os

# PySide import switch
try:
    from PySide import QtGui as QtWidgets
    from PySide import QtGui
    from PySide import QtCore
except ImportError as error:
    from PySide2 import QtWidgets
    from PySide2 import QtGui
    from PySide2 import QtCore


def set_icon(widget, path):
    """Update the icon of the given widget to the given path.

    Args:
        widget (QtWidgets.QWidget): Widget to update the icon on.
        path (str): Absolute path of icon to update to.

    """
    widget.setIcon(QtGui.QIcon(path))


def load_icons():
    """Load all icons from the icons folder.

    This scans the icons directory and creates an icon dictionary using the
    file names without extension as keys.

    Returns:
        dict: Dictionary of icons in the format:
            {
                "icon1": "path1",
                "icon2": "path2",
                "icon3": "path3",
                ...
            }

    """
    this_dir = os.path.dirname(__file__)
    dir_icon = os.path.join(this_dir, "..", "icons")
    dir_icon = os.path.normpath(dir_icon)

    icons = {}
    for file_ in os.listdir(dir_icon):
        name = os.path.splitext(file_)[0]
        path = os.path.join(dir_icon, file_)
        icons[name] = path

    return icons


def set_style_sheet(widget, style="styles.qss"):
    """Apply css style sheet to given widget.

    Args:
        widget (QtWidgets.QWidget): Widget to apply styles to.
        style (str): Name of styles file to apply.

    """
    this_dir = os.path.join(os.path.dirname(__file__))

    styles = os.path.join(this_dir, "..", "styles", style)
    styles = os.path.normpath(styles)

    with open(styles) as file_:
        widget.setStyleSheet(file_.read())


def move_widget_under_cursor(widget, center_window=False):
    """Move given widget under the current cursor position.

    Args:
        widget (QtWidgets.QWidget): The widget to move to the current cursor
            position.
        center_window (bool, optional): If True perform an additional centering
            of the widget to the current cursor location, otherwise leave the
            widget's top left corner at the current cursor position.

    """
    position = QtGui.QCursor().pos()

    if center_window:
        window = QtCore.QPoint(0.5 * widget.width(), 0.5 * widget.height())
        position = position - window

    widget.move(position)


def flush_layout(layout):
    """Remove all widgets from the given layout.

    Args:
        layout (QtWidgets.QLayout): Layout to remove all widgets from.

    """
    for i in reversed(range(layout.count())):
        layout.itemAt(i).widget().deleteLater()


def set_range_style(slider, value, knob):
    """Set style based on current value.

    We set a special style when exceeding the range to indicate the user
    went over the knobs range.

    Args:
        slider (smartEdit.widgets.slider): Slider to apply with styles.
        value (int): The current value of the slider
        knob (nuke.Knob): The underlying knob to as for bounds.

    """
    if value < knob.minimum() or value > knob.maximum():
        slider.setProperty("style", "exceed_range")
    else:
        slider.setProperty("style", "standard")

    set_style_sheet(slider)
