"""Main window for this package."""

# PySide import switch
try:
    from PySide import QtGui as QtWidgets
    from PySide import QtGui
    from PySide import QtCore
except ImportError as error:
    from PySide2 import QtWidgets
    from PySide2 import QtGui
    from PySide2 import QtCore

# Import local modules
from smartEdit import helper
from smartEdit.mvc import gui_helper
from smartEdit.mvc import settings
from smartEdit.mvc import widgets


# Icons for this package.
_ICONS = gui_helper.load_icons()


class Main(QtWidgets.QWidget):
    """Main window that contains the widgets to drive the node's knobs."""

    key_pressed = QtCore.Signal(str)
    key_released = QtCore.Signal(str)

    def __init__(self):
        """Initialize the Main instance."""
        super(Main, self).__init__()

        self.setup()
        self.build_ui()

        self.hotkey_config = helper.load_settings()["hotkey"].upper()
        self.hotkey = getattr(QtCore.Qt, "Key_{}".format(self.hotkey_config))

    def setup(self):
        """Setup ui."""
        self.setMinimumWidth(500)
        self.setProperty("style", "main")

        # Set window flags.
        frameless = QtCore.Qt.FramelessWindowHint
        always_on_top = QtCore.Qt.WindowStaysOnTopHint

        if helper.load_settings()["frameless"]:
            self.setWindowFlags(always_on_top | frameless)
        else:
            self.setWindowFlags(always_on_top)

        self.setAcceptDrops(True)

    def build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.settings = settings.Settings()
        self.settings.setVisible(False)
        self.button_pin = QtWidgets.QToolButton()
        self.button_settings = QtWidgets.QToolButton()
        gui_helper.set_icon(self.button_settings, _ICONS["settings"])
        self.button_plus = widgets.PlusButton()
        gui_helper.set_icon(self.button_plus, _ICONS["plus"])

        # This is a hack to keep the main window in the correct state of being
        # a singleton and fix Nuke's internal behaviour of executing a callable
        # multiple times when holding the specific hotkey. In order to
        # accomplish that we create this QLineEdit and set the focus to this
        # widget. We set its height to 0 and and also remove its border so that
        # it wont get shown. Keep in mind that simply setting the widget
        # invisible causes the widget to loose focus. That is why we need to
        # use this style hack. We can also not simply set the focus to some
        # other widget type like a button, because this will not hold the
        # focus and would cause Nuke to create multiple views being stacked on
        # top of each other. This hack seems to be a reliable way to accomplish
        # turning off Nuke's behavior of executing a callable multiple when
        # holding a hotkey that invokes the callable.
        self.input_keep = QtWidgets.QLineEdit()
        int_validator = QtGui.QIntValidator()
        self.input_keep.setValidator(int_validator)
        self.input_keep.setProperty("style", "input_keep")

    def create_layouts(self):
        """Create layouts."""
        self.layout_master = QtWidgets.QVBoxLayout()
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_quickscripts = QtWidgets.QVBoxLayout()

        self.layout_widgets = QtWidgets.QVBoxLayout()
        self.layout_bottom = QtWidgets.QHBoxLayout()

        self.layout_bottom.addWidget(self.button_pin)
        self.layout_bottom.addWidget(self.button_settings)
        self.layout_bottom.addStretch()
        self.layout_bottom.addWidget(self.button_plus)

        self.layout_main.addLayout(self.layout_quickscripts)
        self.layout_main.addLayout(self.layout_widgets)
        self.layout_main.addStretch()
        self.layout_main.addLayout(self.layout_bottom)
        self.layout_main.addWidget(self.input_keep)

        self.group_main = QtWidgets.QWidget()
        self.group_main.setLayout(self.layout_main)

        self.layout_master.addWidget(self.settings)
        self.layout_master.addWidget(self.group_main)

        self.setLayout(self.layout_master)

        gui_helper.set_style_sheet(self)

    def keyPressEvent(self, event, *args):
        """Catch keyPressEvent to execute custom events."""
        super(Main, self).keyPressEvent(event)

        if event.isAutoRepeat():
            return

        if event.key() == QtCore.Qt.Key_Escape:
            self.close()

        # This is needed otherwise when holding the launch shortcut Z and
        # clicking a widget, the main window looses focus and this emits a
        # keyReleaseEvent which will create another instance at the clocked
        # location. Here we prevent from this in order to check if the window
        # has focus and if not we will not perform the launch of a new window
        # but will instead keep the current window instance open.
        if not self.hasFocus():
            return

    def keyReleaseEvent(self, event, *args):
        """Catch keyReleaseEvent to execute custom events."""
        super(Main, self).keyReleaseEvent(event)

        if event.isAutoRepeat():
            return

        if event.key() == self.hotkey:
            self.key_released.emit("launch_hotkey")
