"""Slider with int field."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError as error:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartEdit import helper
from smartEdit.mvc import gui_helper
from smartEdit.widgets.base import KnobBase


class Slider(KnobBase):
    """Slider with int field."""

    # Smallest maximum value before start to generate floating sub value in
    # between for the slider.
    SMALLEST_MAX_VAL = float(
        pow(10, helper.load_settings()["slider_precision"]))

    def create_widgets(self):
        """Create widgets."""
        self.label = QtWidgets.QLabel(self.knob.name())
        self.input = QtWidgets.QLineEdit()
        self.slider = QtWidgets.QSlider()
        self.input_increment = QtWidgets.QLineEdit()
        self.plus = QtWidgets.QToolButton()
        self.minus = QtWidgets.QToolButton()

        self.knob_minimum = self.knob.minimum()
        self.knob_maximum = self.knob.maximum()

    def setup_widgets(self):
        """Setup widgets."""
        self.input.setText(str(self.knob.value()))

        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.slider.setMinimum(self.knob.minimum())
        self.slider.setMaximum(self.knob.maximum())

        self.slider.setRange(0, self.knob.maximum())

        gui_helper.set_range_style(self.slider, self.knob.value(), self.knob)

        gui_helper.set_icon(self.plus, self.icons["plus"])
        gui_helper.set_icon(self.minus, self.icons["minus"])

        if self.knob_maximum < self.SMALLEST_MAX_VAL:
            self.slider.setMaximum(self.knob.maximum() * self.SMALLEST_MAX_VAL)

        # In case we are dealing with a channel split knob, the knob value will
        # be of list type. In that case we don't want to set the slider value.
        if not isinstance(self.knob.value(), list):
            self.slider.setValue(self.knob.value())
            if self.knob_maximum < self.SMALLEST_MAX_VAL:
                self.slider.setValue(self.knob.value() * self.SMALLEST_MAX_VAL)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main.addWidget(self.label)
        self.layout_main.addWidget(self.input, 4)
        self.layout_main.addWidget(self.slider, 10)

        # If advanced mode show increment/ decrement widgets to increment /
        # decrement the current value by a specific value.
        if self.advanced:
            self.layout_main.addWidget(self.input_increment, 2)
            self.layout_main.addWidget(self.minus)
            self.layout_main.addWidget(self.plus)

    def create_signals(self):
        """Create signals."""
        self.slider.valueChanged.connect(lambda: self.update("slider"))
        self.input.returnPressed.connect(lambda: self.update("input"))
        self.plus.clicked.connect(lambda: self.increment())
        self.minus.clicked.connect(lambda: self.increment(increase=False))

    def increment(self, increase=True):
        """Increment/ decrement the current value by specified value.

        Args:
            increase (bool, optional): If True then increment, otherwise
                decrement.

        """
        sign = "+" if increase else "-"

        try:
            expression = ("{}{}{}".format(self.input.text(), sign,
                                            self.input_increment.text()))
            updated = float(eval(expression))

        except (NameError, SyntaxError):
            # The user entered some characters into the int field so that we
            # cannot evaluate. In that case we don't want to process anything
            # here.
            return

        self.input.setText(str(updated))
        gui_helper.set_range_style(self.slider, updated, self.knob)
        self.knob.setValue(updated)

        self.slider.setValue(updated)

    def update(self, who):
        """Update the slider and the input.

        Args:
            who (str): Knob identifier that was invoked.

        """
        if who == "slider":
            if self.knob_maximum < self.SMALLEST_MAX_VAL:
                value = self.slider.value() / self.SMALLEST_MAX_VAL
            else:
                value = self.slider.value()
            self.input.setText(str(value))

        else:
            try:
                value = float(self.input.text())
            except ValueError:
                return
            self.slider.valueChanged.disconnect()
            self.slider.setValue(value)
            self.create_signals()

        self.knob.setValue(value)
        gui_helper.set_range_style(self.slider, value, self.knob)
